/*
 * arminstruction.cc
 *
 *  Created on: 2009. 10. 25.
 *      Author: Administrator
 */

#include "arminstruction.h"

void ARMinstruction::Decode(uint32_t binary)
{
	codeId = binary >> 26 & 0x3;
	switch(codeId) {
	case 0: _decode0(binary); break;
	case 1: _decode1(binary); break;
	case 2: _decode2(binary); break;
	case 3: _decode3(binary); break;
	default:
		return;
	}
}

void ARMinstruction::_decode0(uint32_t binary)
{
	/*
	 * opcode 0에서 가능한 명령어 분류
	 * ARM instruction manual의 A3-2를 참고.
	 * Data processing immediate shift :
	 *	000 / 4bit가 0
	 * Miscellaneous instructions I
	 *	000 / 10xx / 0 / 4bit가 0
	 * Data processing register shift
	 *	000 / opcode / 7비트 0 / 4비트 1
	 * Miscellaneous instructions II
	 *	000 / 10xx / 0 / 7비트 0 / 4비트 1
	 * Multiplies, Extra load/stores
	 *	000 / 7비트 1 / 4비트 1
	 * Data processing immediate
	 *	001
	 * Undefined instruction
	 *	001 / 10x00
	 * Move immediate to status register
	 *	001 / 10x10
	 */
	// 가장 먼저 할 일은 cond가 1111인가를 살피는 것
	cond = binary >> 28 & 0xF;
	if( cond == 0xF ) {
		printf("헐... unconditional instruction 이네여\n");
	}

	// 그 다음 25비트가 0인지 1인지 따지자
	if( (binary >> 25 & 1) == 0 ) { // 000
		// 000은 4비트가 0인 경우와 1인 경우로 나뉜다.
		if( (binary >> 4 & 1) == 0 ) { // 4비트가 0
			// 이건 또 24~20비트가 10xx0이냐, 아니냐로 구분된다.
			if( (binary >> 20 & 0x19) == 16 ) { // 10xx0: Miscellaneous instructions I
				printf("Miscellaneous instructions I 이네여\n");
			}
			else { // Data processing immediate shift
				printf("Data processing immediate shift네여\n");
				_decodeDataProcessingImmediateShift(binary);
			}
		}
		else { // 4비트가 1
		}
	} // 000
	else { // 001
	} // 001
} // end of _decode0

void ARMinstruction::_decode1(uint32_t binary)
{
	printf("알수없는 binary %x\n", binary);
}

void ARMinstruction::_decode2(uint32_t binary)
{
	printf("알수없는 binary %x\n", binary);
}

void ARMinstruction::_decode3(uint32_t binary)
{
	printf("알수없는 binary %x\n", binary);
}

// 이름이 너무 길어서 슬픈 함수들

void ARMinstruction::_decodeDataProcessingImmediateShift(uint32_t binary)
{
	// cond(4) 000 opcode(4) S(1) Rn(4) Rd(4) shift amount(5) shift(2) 0 Rm(4)
	// mov 같은 경우 설명에 따르면 25비트는 i bit지만 이 함수의 호출 구조상-_-
	// 여기서는 무조건 0이다.
	ibit = binary >> 25 & 0x1; // 그냥 0
	// opcode부터 해독하자.
	// opcode 목록
	// 0xD: mov
	opcode = binary >> 21 & 0xF;
	sbit = binary >> 20 & 0x1;
	rn_num = binary >> 16 & 0xF;
	rd_num = binary >> 12 & 0xF;
	shift_amount = binary >> 7 & 0x1F;
	shift = binary >> 5 & 0x3;
	rm_num = binary & 0xF;
	printf("opcode = 0x%x\n", opcode);

	switch(opcode) {
		case 0xD: type = MOV; break;
		default: type = UNKNOWN;
	}
}

string cmdString(uint8_t opcode)
{
	switch(opcode) {
		case 0xD: return "mov";
		default: return "unknown";
	}
}

